'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _gitPromise = require('git-promise');

var _gitPromise2 = _interopRequireDefault(_gitPromise);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _findRemoteAlias = require('./findRemoteAlias');

var _findRemoteAlias2 = _interopRequireDefault(_findRemoteAlias);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ensureCleanLocalCopy() {
    return (0, _gitPromise2.default)('status -s').then(function (status) {
        if (status.trim() !== '') {
            throw new Error('Local copy is not clean');
        }
    });
}

function ensureMasterBranch() {
    return (0, _gitPromise2.default)('rev-parse --abbrev-ref HEAD').then(function (branchName) {
        if (branchName.trim() !== 'master') {
            throw new Error('Not on master branch');
        }
    });
}

function fetchRemote(remoteAlias) {
    return (0, _gitPromise2.default)('fetch ' + remoteAlias);
}

function ensureLocalIsEqualToRemote(remoteAlias) {
    var remoteBranch = remoteAlias + '/master';

    return (0, _gitPromise2.default)('rev-list --left-right master...' + remoteBranch).then(function (result) {
        var commits = result.split('\n');

        var commitsAhead = 0;
        var commitsBehind = 0;

        commits.forEach(function (commit) {
            if (commit.trim().length > 0) {
                if (commit.indexOf('>') === 0) {
                    commitsBehind += 1;
                } else {
                    commitsAhead += 1;
                }
            }
        });

        if (commitsAhead > 0 || commitsBehind > 0) {
            // eslint-disable-next-line max-len
            var errorMessage = 'Local git master branch is ' + commitsAhead + ' commits ahead and ' + commitsBehind + ' commits behind of ' + remoteBranch;

            throw new Error(errorMessage);
        }
    });
}

function ensureCleanLocalGitState(githubRepo) {
    return _bluebird2.default.try(ensureCleanLocalCopy).then(ensureMasterBranch).then(_findRemoteAlias2.default.bind(null, githubRepo)).tap(fetchRemote).then(ensureLocalIsEqualToRemote);
}

exports.default = ensureCleanLocalGitState;