'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _gitPromise = require('git-promise');

var _gitPromise2 = _interopRequireDefault(_gitPromise);

var _gitUrlParse = require('git-url-parse');

var _gitUrlParse2 = _interopRequireDefault(_gitUrlParse);

var _ramda = require('ramda');

var _ramda2 = _interopRequireDefault(_ramda);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isSameGitUrl(gitUrlA, gitUrlB) {
    var parsedUrlA = (0, _gitUrlParse2.default)(gitUrlA);
    var parsedUrlB = (0, _gitUrlParse2.default)(gitUrlB);
    var pathA = parsedUrlA.pathname.replace(/\.git$/, '');
    var pathB = parsedUrlB.pathname.replace(/\.git$/, '');

    return parsedUrlA.resource === parsedUrlB.resource && pathA === pathB;
}

function getGitUrl(githubRepo) {
    return 'git://github.com/' + githubRepo + '.git';
}

function findRemoteAlias(githubRepo) {
    var gitRemote = getGitUrl(githubRepo);

    return (0, _gitPromise2.default)('remote -v').then(function (output) {
        var remotes = output.split('\n').map(function (remote) {
            var tokens = remote.split(/\s/);

            return {
                alias: tokens[0],
                url: tokens[1]
            };
        });

        var matchedRemote = _ramda2.default.find(function (remote) {
            return remote.url && isSameGitUrl(gitRemote, remote.url);
        }, remotes);

        if (!matchedRemote) {
            throw new Error('This local git repository doesn’t have a remote pointing to ' + gitRemote);
        }

        return matchedRemote.alias;
    });
}

exports.default = findRemoteAlias;