'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _gitPromise = require('git-promise');

var _gitPromise2 = _interopRequireDefault(_gitPromise);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _semver = require('semver');

var _semver2 = _interopRequireDefault(_semver);

var _getPullRequestLabel = require('./getPullRequestLabel');

var _getPullRequestLabel2 = _interopRequireDefault(_getPullRequestLabel);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getLatestVersionTag() {
    return (0, _gitPromise2.default)('tag --list').then(function (result) {
        var tags = result.split('\n');
        var versionTags = tags.filter(_semver2.default.valid);
        var orderedVersionTags = versionTags.sort(_semver2.default.compare);

        return orderedVersionTags[orderedVersionTags.length - 1];
    });
}

function getPullRequests(fromTag) {
    return (0, _gitPromise2.default)('log --no-color --pretty=format:"%s (%b)" --merges ' + fromTag + '..HEAD').then(function (result) {
        var mergeCommits = result.replace(/[\r\n]+\)/g, ')').split('\n');

        return mergeCommits.filter(function (commit) {
            return commit.indexOf('Merge pull request') === 0;
        }).map(function (pullRequest) {
            var pattern = /^Merge pull request #(\d+) from (.*?) \((.*)\)$/;
            var matches = pullRequest.match(pattern);

            return {
                id: matches[1],
                title: matches[3]
            };
        });
    });
}

function extendWithLabel(githubRepo, pullRequests) {
    var promises = pullRequests.map(function (pullRequest) {
        return (0, _getPullRequestLabel2.default)(githubRepo, pullRequest.id).then(function (label) {
            return {
                id: pullRequest.id,
                title: pullRequest.title,
                label: label
            };
        });
    });

    return _bluebird2.default.all(promises);
}

function getMergedPullRequests(githubRepo) {
    return getLatestVersionTag().then(getPullRequests).then(extendWithLabel.bind(null, githubRepo));
}

exports.default = getMergedPullRequests;